#if !defined(ptf_err_h)
//! \internal header guard
#define ptf_err_h

#if !defined(ptf_h)
#include "ptf.h"
#endif

//! \addtogroup PtfLibrary The PTF File Library
//! @{

//! ptf_err is used in the API to return status information. It is not stored 
//! on disk.
enum ptf_err {
  peSuccess,            //!< Everything went fine.
  peFileNotFound,       //!< The indicated file was not found.
  peInvalidFile,        //!< The indicated path is not a PTF file.
  peIoError,            //!< Some error when attempting to read/write the file.
  peInvalidVersion,     //!< The indicated file is of the wrong version.
  pePermissionDenied,   //!< The indicated file is not accessible, or attempt to write to read-only file.
  peInvalidArguments,   //!< The arguments to the function are wrong.
  peTooBigForMachine,   //!< The data in question is too big for the current machine (i e, needs 64 bit memory).
  peReadOutOfBounds,    //!< Attempt to read past the end of a record.
  peInvalidSequence,    //!< The attempted function is not currently valid.
  peReadOnly,           //!< The file is not open for writing.
  peException,          //!< C++ exception occurred while executing -- probably out of memory.
  peOutOfMemory,        //!< One of those rare occasions where out of memory is not an exception.
  peTypeNotFound,       //!< Request for a type that doesn't exist.
  peRecordNotFound,     //!< Record index is out of range, or has been deleted.
  peNotSupported,       //!< Feature not implemented (for example, encryption method).
  peDuplicateName,      //!< Name (or ID) must be unique, and isn't.
  peNameNotFound,       //!< The requested name maps to nothing. "name" can mean an ID or a string name.
  peWrongFile,          //!< The operation was attempted on the wrong PTF file (when dealing with cross-file references).
};
typedef enum ptf_err ptf_Err;

//! Get a user-readable string (in English) describing the error.
//! \param err The error to describe.
//! \return A description of the error. Will never return NULL.
//! Will return a generic description for error values out of range or 
//! unknown.
PTF_API char const * ptf_err_get_text(ptf_err err);

#if !defined(ptf_err_h)
}
#endif

//! @}
#endif  //  ptf_err_h
