
#include "ptf_bookmark.h"

#include <string.h>

// original bookmark version - without ori
struct old_ptf_bookmark_v1 {
  ptf_name                    name;
  ptf_geopos                  pos;
  double                      radius;
};



PTF_API ptf_err ptf_file_create_bookmark( ptf_file_handle file, ptf_bookmark const * bookmark, ptf_fixed_ref * o_ref )
{
  try {
    ptf_record_ref rref;
    memset(&rref, 0, sizeof(rref));
    rref.type = ptBookmark;

    ptf_err err = ptf_file_new_complete_record(file, ptBookmark, (void const *) bookmark, sizeof(ptf_bookmark), &rref);
    if (err != peSuccess) return err;
    if ( o_ref ) {
      o_ref->index = rref.index;
    }
    return err;
  }
  catch (...) {
    return peException;
  }
}

PTF_API ptf_err ptf_file_remove_bookmark( ptf_file_handle file, ptf_fixed_ref const &ref )
{
  try {
    ptf_record_ref rref;
    memset(&rref, 0, sizeof(rref));
    rref.type = ptBookmark;
    rref.index = ref.index;

    ptf_err err = ptf_file_remove_record(file, &rref);
    return err;
  }
  catch(...) {
    return peException;
  }
}

PTF_API ptf_err ptf_file_get_bookmark( ptf_file_handle file, ptf_fixed_ref const &ref, ptf_bookmark * bookmark )
{
  try {
    ptf_record_ref rref;
    memset(&rref, 0, sizeof(rref));
    rref.type = ptBookmark;
    rref.index = ref.index;

    ptf_64 recSize = 0;
    ptf_err err = ptf_file_get_record_size( file, &rref, &recSize );
    if (err != peSuccess) return err;
    if ( recSize > sizeof(ptf_bookmark) ) {
      // stored record has unexpected size - assume this is a format version issue
      return peInvalidVersion;
    }

    // backwards compatibility
    switch ( recSize ) {
      case sizeof(old_ptf_bookmark_v1):
        // fill in default ori
        bookmark->ori.x = 0.0f;
        bookmark->ori.y = 0.0f;
        bookmark->ori.z = 0.0f;
        bookmark->ori.w = 1.0f;
        break;
    }

    err = ptf_file_read_record( file, &rref, 0, bookmark, recSize );

    return err;
  }
  catch(...) {
    return peException;
  }
}

PTF_API ptf_err ptf_file_set_bookmark( ptf_file_handle file, ptf_fixed_ref const &ref, ptf_bookmark const * bookmark )
{
  try {
    ptf_record_ref rref;
    memset(&rref, 0, sizeof(rref));
    rref.type = ptBookmark;
    rref.index = ref.index;
    ptf_err err = ptf_file_replace_record(file, &rref);
    if (err != peSuccess) return err;
    err = ptf_file_write_record(file, (void const *) bookmark, sizeof(ptf_bookmark));
    if (err != peSuccess) return err;
    err = ptf_file_commit_record(file, &rref);
    return err;
  }
  catch(...) {
    return peException;
  }
}

